@@ -4,9 +4,12 @@
 For usage, see README.md.
 '''
 
+from __future__ import print_function
+from binaryornot.check import is_binary
 import logging
 import mimetypes
 import os
+import platform
 import requests
 import socket
 import sys
@@ -14,11 +17,14 @@ from threading import Timer
 import time
 import webbrowser
 
-from flask import (Flask, render_template, send_from_directory,
+from flask import (Flask, render_template, send_from_directory, send_file,
                    request, jsonify, Response)
 
-import util
-import argparser
+from webdiff import diff
+from webdiff import util
+from webdiff import argparser
+
+VERSION = '0.15.0'
 
 
 def determine_path():
@@ -29,8 +35,8 @@ def determine_path():
             root = os.path.realpath (root)
         return os.path.dirname (os.path.abspath (root))
     except:
-        print "I'm sorry, but something is wrong."
-        print "There is no __file__ variable. Please contact the author."
+        print("I'm sorry, but something is wrong.")
+        print("There is no __file__ variable. Please contact the author.")
         sys.exit()
 
 def is_hot_reload():
@@ -39,17 +45,16 @@ def is_hot_reload():
 
 
 class Config:
-    pass
-    #TESTING=True  # not exactly sure what this does...
+    TESTING=False  # not exactly sure what this does...
+    JSONIFY_PRETTYPRINT_REGULAR=False
 
 app = Flask(__name__)
 app.config.from_object(Config)
 app.config.from_envvar('WEBDIFF_CONFIG', silent=True)
 
-A_DIR = None
-B_DIR = None
 DIFF = None
 PORT = None
+HOSTNAME = 'localhost'
 
 if app.config['TESTING'] or app.config['DEBUG']:
     handler = logging.StreamHandler()
@@ -75,65 +80,95 @@ def update_last_request_ms():
     LAST_REQUEST_MS = time.time() * 1000
 
 
+def error(code, message):
+    e = {"code": code, "message": message}
+    response = jsonify(e)
+    response.status_code = 400
+    return response
+
+
 @app.route("/<side>/get_contents", methods=['POST'])
 def get_contents(side):
-    assert side in ('a', 'b')
+    if side not in ('a', 'b'):
+        return error('invalid side', 'Side must be "a" or "b", got %s' % side)
+
+    # TODO: switch to index? might be simpler
     path = request.form.get('path', '')
     if not path:
-        e = {"code": "incomplete",
-             "message": "Incomplete request (need path)"}
-        response = jsonify(e)
-        response.status_code = 400
-        return response
+        return error('incomplete', 'Incomplete request (need path)')
+
+    idx = diff.find_diff_index(DIFF, side, path)
+    if idx is None:
+        return error('not found', 'Invalid path on side %s: %s' % (side, path))
+
+    d = DIFF[idx]
+    abs_path = d.a_path if side == 'a' else d.b_path
 
     try:
-        abs_path = os.path.join(A_DIR if side == 'a' else B_DIR,
-                                os.path.normpath(path))  # / --> \ on windows
-        is_binary = util.is_binary_file(abs_path)
-        if is_binary:
+        if is_binary(abs_path):
             size = os.path.getsize(abs_path)
             contents = "Binary file (%d bytes)" % size
         else:
             contents = open(abs_path).read()
         return Response(contents, mimetype='text/plain')
     except Exception:
-        e = {"code": "read-error",
-             "message": "Unable to read %s" % abs_path}
-        response = jsonify(e)
-        response.status_code = 400
-        return response
+        return error('read-error', 'Unable to read %s' % abs_path)
 
 
 @app.route("/<side>/image/<path:path>")
 def get_image(side, path):
-    assert side in ('a', 'b')
+    if side not in ('a', 'b'):
+        return error('invalid side', 'Side must be "a" or "b", got %s' % side)
+
+    # TODO: switch to index? might be simpler
     if not path:
-        e = {"code": "incomplete",
-             "message": "Incomplete request (need path)"}
-        response = jsonify(e)
-        response.status_code = 400
-        return response
+        return error('incomplete', 'Incomplete request (need path)')
 
     mime_type, enc = mimetypes.guess_type(path)
-    if not mime_type.startswith('image/') or enc is not None:
-        e = {"code": "wrongtype",
-             "message": "Requested file of type (%s, %s) as image" % (
-                 mime_type, enc)}
-        response = jsonify(e)
-        response.status_code = 400
-        return response
+    if not mime_type or not mime_type.startswith('image/') or enc is not None:
+        return error('wrongtype', 'Requested file of type (%s, %s) as image' % (
+                 mime_type, enc))
+
+    idx = diff.find_diff_index(DIFF, side, path)
+    if idx is None:
+        return error('not found', 'Invalid path on side %s: %s' % (side, path))
+
+    d = DIFF[idx]
+    abs_path = d.a_path if side == 'a' else d.b_path
 
     try:
-        abs_path = os.path.join(A_DIR if side == 'a' else B_DIR,
-                                os.path.normpath(path))  # / --> \ on windows
-        contents = open(abs_path).read()
+        contents = open(abs_path, mode='rb').read()
         return Response(contents, mimetype=mime_type)
     except Exception:
-        e = {"code": "read-error",
-             "message": "Unable to read %s" % abs_path}
-        response = jsonify(e)
-        response.status_code = 400
-        return response
+        return error('read-error', 'Unable to read %s' % abs_path)
+
+
+@app.route("/pdiff/<int:idx>")
+def get_pdiff(idx):
+    idx = int(idx)
+    d = DIFF[idx]
+    try:
+        _, pdiff_image = util.generate_pdiff_image(d.a_path, d.b_path)
+        dilated_image = util.generate_dilated_pdiff_image(pdiff_image)
+    except util.ImageMagickNotAvailableError:
+        return 'ImageMagick is not available', 501
+    except util.ImageMagickError as e:
+        return 'ImageMagick error %s' % e, 501
+    return send_file(dilated_image)
+
+
+@app.route("/pdiffbbox/<int:idx>")
+def get_pdiff_bbox(idx):
+    idx = int(idx)
+    d = DIFF[idx]
+    try:
+        _, pdiff_image = util.generate_pdiff_image(d.a_path, d.b_path)
+        bbox = util.get_pdiff_bbox(pdiff_image)
+    except util.ImageMagickNotAvailableError:
+        return 'ImageMagick is not available', 501
+    except util.ImageMagickError as e:
+        return 'ImageMagick error %s' % e, 501
+    return jsonify(bbox)
 
 
 # Show the first diff by default
@@ -145,9 +180,17 @@ def index():
 @app.route("/<int:idx>")
 def file_diff(idx):
     idx = int(idx)
+    pairs = diff.get_thin_list(DIFF)
     return render_template('file_diff.html',
                            idx=idx,
-                           pairs=DIFF)
+                           has_magick=util.is_imagemagick_available(),
+                           pairs=pairs)
+
+
+@app.route('/thick/<int:idx>')
+def thick_diff(idx):
+    idx = int(idx)
+    return jsonify(diff.get_thick_dict(DIFF[idx]))
 
 
 @app.route('/favicon.ico')
@@ -168,28 +211,30 @@ def seriouslykill():
 
 @app.route('/kill', methods=['POST'])
 def kill():
+    global PORT
     if 'STAY_RUNNING' in app.config:
         return 'Will stay running.'
 
     last_ms = LAST_REQUEST_MS
     def shutdown():
         if LAST_REQUEST_MS <= last_ms:  # subsequent requests abort shutdown
-            requests.post('http://localhost:%d/seriouslykill' % PORT)
+            requests.post('http://%s:%d/seriouslykill' % (HOSTNAME, PORT))
         else:
             pass
 
     Timer(0.5, shutdown).start()
-    
+
     return 'Shutting down...'
 
 
 def open_browser():
     global PORT
+    global HOSTNAME
     if not 'NO_OPEN_BROWSER' in app.config:
         if is_hot_reload():
             log.debug('Skipping browser open on reload')
         else:
-            webbrowser.open_new_tab('http://localhost:%s' % PORT)
+            webbrowser.open_new_tab('http://%s:%s' % (HOSTNAME, PORT))
 
 
 def usage_and_die():
@@ -211,7 +256,7 @@ def pick_a_port(args):
     return port
 
 
-def abs_path(path):
+def abs_path_from_rel(path):
     '''Changes relative paths to be abs w/r/t/ the original cwd.'''
     if os.path.isabs(path):
         return path
@@ -219,27 +264,39 @@ def abs_path(path):
         return os.path.join(os.getcwd(), path)
 
 
+def is_webdiff_from_head():
+    '''Was webdiff invoked as `git webdiff` with no other non-flag args?'''
+    return os.environ.get('WEBDIFF_FROM_HEAD') is not None
+
+
 def run():
-    global A_DIR, B_DIR, DIFF, PORT
+    global DIFF, PORT, HOSTNAME
     try:
-        parsed_args = argparser.parse(sys.argv[1:])
+        parsed_args = argparser.parse(sys.argv[1:], VERSION)
     except argparser.UsageError as e:
-        sys.stderr.write('Error: %s\n\n' % e.message)
+        sys.stderr.write('Error: %s\n\n' % e)
         usage_and_die()
 
-    A_DIR, B_DIR, DIFF = util.diff_for_args(parsed_args)
+    DIFF = argparser.diff_for_args(parsed_args)
 
     if app.config['TESTING'] or app.config['DEBUG']:
-        sys.stderr.write('Diffing:\nA: %s\nB: %s\n\n' % (A_DIR, B_DIR))
+        sys.stderr.write('Diff:\n%s' % DIFF)
 
     PORT = pick_a_port(parsed_args)
 
-    sys.stderr.write('''Serving diffs on http://localhost:%s
+    if app.config.get('USE_HOSTNAME'):
+        _hostname = platform.node()
+        # platform.node will return empty string if it can't find the hostname
+        if not _hostname:
+            sys.stderr.write('Warning: hostname could not be determined')
+        else:
+            HOSTNAME = _hostname
+
+    sys.stderr.write('''Serving diffs on http://%s:%s
 Close the browser tab or hit Ctrl-C when you're done.
-''' % PORT)
-    logging.info('Diff: %r', DIFF)
+''' % (HOSTNAME, PORT))
     Timer(0.1, open_browser).start()
-    app.run(host='0.0.0.0', port=PORT)
+    app.run(host=HOSTNAME, port=PORT)
 
 
 if __name__ == '__main__':
