@@ -3,30 +3,26 @@
 This will create symlinks or clone git repos as needed.
 """
 
-import atexit
+# Use this PR for testing to see all four types of change at once:
+# https://github.com/danvk/test-repo/pull/2/
+
 from collections import OrderedDict
-import errno
 import os
 import re
 import subprocess
 import sys
-import tempfile
 
 from github import Github, UnknownObjectException
 
+from webdiff.util import memoize
 
-temp_dirs = []
-
-_github_memo = None
-def _github():
-    global _github_memo
-    if _github_memo: return _github_memo
 
+@memoize
+def github():
+    '''Returns a GitHub API object with auth, if it's available.'''
     def simple_fallback(message=None):
-        global _github_memo
         if message: sys.stderr.write(message + '\n')
-        _github_memo = Github()
-        return _github_memo
+        return Github()
 
     github_rc = os.path.join(os.path.expanduser('~'), '.githubrc')
     if os.path.exists(github_rc):
@@ -41,57 +37,25 @@ def _github():
                 k, v = line.split(': ', 1)
                 kvs[k] = v
 
-            if not kvs.get('user.login'):
+            login = kvs.get('user.login')
+            if not login:
                 return simple_fallback('.githubrc missing user.login. Using anonymous API access.')
-            if not kvs.get('user.password'):
-                return simple_fallback('.githubrc missing user.password. Using anonymous API access.')
-            _github_memo = Github(kvs['user.login'], kvs['user.password'])
+
+            password = kvs.get('user.password')
+            token = kvs.get('user.token')
+
+            if password and token:
+                raise OnlyPasswordOrToken('Only specify user.token or user.password '
+                                          'in your .githubrc file (got both)')
+
+            auth = token or password
+
+            if not auth:
+                return simple_fallback('.githubrc has neither user.password nor user.token.'
+                                       'Using anonymous API access.')
+            return Github(login, auth)
     else:
-        _github_memo = Github()
-    return _github_memo
-
-
-def put_in_dir(dirname, filename, contents):
-    """Puts contents into filename in dirname.
-    
-    This creates intermediate directories as needed, e.g. if filename is a/b/c.
-    """
-    d = os.path.join(dirname, os.path.dirname(filename))
-    try:
-        os.makedirs(d)
-    except OSError, e:
-        # be happy if someone already created the path
-        if e.errno != errno.EEXIST:
-            raise
-    open(os.path.join(dirname, filename), 'w').write(contents)
-
-
-def fetch_pull_request(owner, repo, num):
-    """Pull down the pull request into two local directories.
-
-    Returns before_dir, after_dir.
-    """
-    sys.stderr.write('Loading pull request %s/%s#%s from github...\n' % (owner, repo, num))
-    g = _github()
-    pr = g.get_user(owner).get_repo(repo).get_pull(num)
-    base_repo = pr.base.repo
-    head_repo = pr.head.repo
-    files = list(pr.get_files())
-
-    a_dir, b_dir = tempfile.mkdtemp(), tempfile.mkdtemp()
-    temp_dirs.extend([a_dir, b_dir])
-
-    for f in files:
-        sys.stderr.write('  %s...\n' % f.filename)
-        if f.status == 'modified' or f.status == 'deleted':
-            contents = base_repo.get_file_contents(f.filename, pr.base.sha).decoded_content
-            put_in_dir(a_dir, f.filename, contents)
-        if f.status == 'modified' or f.status == 'added':
-            contents = head_repo.get_file_contents(f.filename, pr.head.sha).decoded_content
-            put_in_dir(b_dir, f.filename, contents)
-        # f.status == 'moved'?
-
-    return a_dir, b_dir
+        return Github()
 
 
 class NoRemoteError(Exception):
@@ -100,6 +64,9 @@ class NoRemoteError(Exception):
 class UnknownPullRequestError(Exception):
     pass
 
+class OnlyPasswordOrToken(Exception):
+    pass
+
 
 def get_pr_repo(num):
     """Returns (owner, repo, num) for the PR number for this git repo.
@@ -113,12 +80,12 @@ def get_pr_repo(num):
                             'directory. Are you in a git repo? Try running '
                             '`git remote -v` to debug.')
 
-    g = _github()
+    g = github()
     for remote in remotes:
         owner = remote['owner']
         repo = remote['repo']
         try:
-            pr = g.get_user(owner).get_repo(repo).get_pull(num)
+            g.get_user(owner).get_repo(repo).get_pull(num)
             return (owner, repo, num)
         except UnknownObjectException:
             pass
@@ -167,13 +134,6 @@ def _parse_remotes(remote_lines):
 
 def _get_remotes():
     remote_lines = subprocess.Popen(
-        ['git', 'remote', '-v'], stdout=subprocess.PIPE).communicate()[0].split('\n')
+        ['git', 'remote', '-v'], stdout=subprocess.PIPE).communicate()[0].decode().split('\n')
     return _parse_remotes(remote_lines)
 
-
-
-@atexit.register
-def _cleanup():
-    """Delete any temporary directories which were created by two_folders()."""
-    #for d in temp_dirs:
-    #    os.removedirs(d)
